<?php
/**
 * Description of ManufacturingAjax
 *
 * @author carlos
 */
class ManufacturingAjax extends RepackRepository
{
    public function __construct()
    {
        parent::__construct();
    }
    
    public function getResponse($request, $options)
    {
        return $this->$request($options);
    }
        
    public function validateDateLots($options)
    {
        $tools = new Tools();
        $lots = $options['lots'];
        $dateArray = explode(" ", $options['date']);
        $date = $dateArray[0];
        
        foreach ($lots as $lot) {
            $lotArray = explode('|', $lot);
            $tipo = $lotArray[0];
            $id = $lotArray[1];

            if ($tipo == 'Manifest') {
                $manifestRepo = new ManifestRepository();
                $manifestData = $manifestRepo->getById($id);
                $operationDate = $manifestData['date'];
            } elseif ($tipo == 'Manufacturing') {
                $repackRepo = new RepackRepository();
                $repackData = $repackRepo->getById($id);
                $operationDate = $repackData['onlyDate'];
            }
            
            $date1 = strtotime($date); #Fecha de lote seleccionado
            $date2 = strtotime($operationDate); #Fecha seleccionada para MFG que se esta registrando

            /*Si $date1 es menor a $date2, manda alerta y no puede continuar*/
            /*Significaria que primero se corrio producto y despues se recibio manifiesto o se hizo mfg anterior*/
            if ($date1 < $date2) {
                $operationDate = $tools->setFormatDateToForm($operationDate);
                $msg = $this->_getTranslation("La fecha de referencia seleccionada, no puede ser mayor a la fecha seleccionada como inicio.");
                $msg1 = $this->_getTranslation("</br></br><strong>Referencia seleccionada:</strong>{%{$lot}%}.");
                $msg2 = $this->_getTranslation("</br><strong>Fecha de referencia:</strong>{%{$operationDate}%}");

                $this->flashmessenger->addMessage(array('danger'=>$msg.$msg1.$msg2));

                return array(
                    'response'=>false,
                    'msg'=>$this->flashmessenger->getRawMessage()
                );
            }
        }
        
        return array(
            'response'=>true
        );
    }

    public function setPalletListByLot(array $options)
    {
        $lots = $options['lot'];
        
        /*$options['lots'] lo agregamos porque se utiliza en validateDateLots*/
        if (!is_array($lots)) {
            $options['lots'] = array($lots=>$lots);
        } else {
            $options['lots'] = $lots;
        }
        
        //$validate = $this->validateDateLots($options);
        $validate = true;
        
        if ($validate['response'] === false) {
            return $validate;
        }
        
        $repoManInputs = new ManufacturingInputTempRepository();
        $repoManInputs->setPalletListByLot($options['lot'], $options['token_form']);
        $pallets = $repoManInputs->getPalletList($options['token_form']);
        
        $details = $this->listPalletsDetails($pallets);
        $json = array(
            'response' => true,
            'palletsDetails' => $details['palletsDetails'],
            'distinctLots'=>$details['distinctLots'],
            'cajasTotal'=>$details['cajasTotal'],
            'palletsTotal'=>$details['palletsTotal'],
            'pesoTotal'=>$details['pesoTotal'],
            'cajasInputTotal'=>$details['cajasInputTotal'],
            'palletsInputTotal'=>$details['palletsInputTotal']
        );

        return $json;
    }
    
    public function listPalletsDetails($pallets)
    {
        $login = new Login();
        $string  = "";
        $cajasTotal = 0;
        $palletsTotal = 0;
        $sumaPesoTotal = 0;
        $cajasInputTotal = 0;
        $palletsInputTotal = 0;
        $tokenForm = null;
        
        if (is_array($pallets) && count($pallets)>0) {
            foreach ($pallets as $item) {
                $tokenForm = $item['token_form'];
                $checked = '';
                $editInventory = '';
                if ($item['status'] == '2') {
                    $checked = 'checked';
                    $editInventory = 'editInputBoxes';
                }
                
                $cajasTotal += $item['quantity_boxes'];
                $palletsTotal += $item['quantity_pallets'];
                $sumaPesoTotal += $item['weight'];

                $cajasInputTotal += $item['quantity_boxes_input'];
                $palletsInputTotal += $item['quantity_pallets_input'];
                
                $lot = str_replace(',', "<br>", $item['lot_concat']);
                $descripcion = str_replace(',', "<br>", $item['descripcion']);
                $variety = str_replace(',', "<br>", $item['varietyName']);
                $size = str_replace(',', "<br>", $item['sizeName']);
                $packing = str_replace(',', "<br>", $item['packingName']);
                $quality = str_replace(',', "<br>", $item['qualityName']);
                $color = str_replace(',', "<br>", $item['colorName']);
                $weight = str_replace(',', "<br>", $item['weight_concat']);
                $boxesQuantity = str_replace(',', "<br>", $item['quantity_boxes_concat']);
                $palletsInputQuantity = str_replace(',', "<br>", $item['quantity_pallets_input_concat']);
                
                $boxesInputQuantity = '';
                $idsArray = explode(',', $item['id_concat']);
                $boxesInputQuantityArray = explode(',', $item['quantity_boxes_input_concat']);
                $lotsArray = explode(',', $item['lot_concat']);
                foreach ($boxesInputQuantityArray as $key => $value) {
                    if ($login->getRole() != '3') {
                        $boxesInputQuantity .= "<a class='$editInventory' data-iddetalle='{$idsArray[$key]}' data-lot='{$item['type']}|{$item['reference']}|{$lotsArray[$key]}'>$value</a><br>";
                    } else {
                        $boxesInputQuantity .= "$value<br>";
                    }
                }
                
                $string .= "<tr> "
                        .  "<td class='text-center'>{$lot}</td>"
                        .  "<td class='text-center'>{$item['id_pallet']}</td>"
                        .  "<td class='text-center'>$descripcion</td>"
                        .  "<td class='text-center'>$variety</td>"
                        .  "<td class='text-center'>$size</td>"
                        .  "<td class='text-center'>$packing</td>"
                        .  "<td class='text-center'>$quality</td>"
                        .  "<td class='text-center'>$color</td>"
                        .  "<td class='text-right'>$weight</td>"
                        .  "<td class='text-right'>$boxesQuantity</td>"
                        .  "<td class='text-right $editInventory'>$boxesInputQuantity</td>"
                        . "<td class='text-right'>$palletsInputQuantity</td>"
                        .  "<td class='text-center'>";
                        
                if ($login->getRole() != '3') {
                    $string .= "<input type='checkbox' value='{$item['status']}' class='_{$item['id_pallet']} _checkbox_setInputPallet' data-iddetalle='{$item['id']}' onChange='setInputPallet(this)' $checked  style='width:27px;height:27px;margin-top:0px'>";
                }
                                
                $string .= "</td>";
                $string .= "</tr>";
            }
        }
        
        $json = array(
                'response' => true,
                'palletsDetails'=>$string,
                'distinctLots'=>$this->setDistinctLots($tokenForm),
                'cajasTotal'=>$cajasTotal,
                'palletsTotal'=>$palletsTotal,
                'pesoTotal'=>$sumaPesoTotal,
                'cajasInputTotal'=>$cajasInputTotal,
                'palletsInputTotal'=>$palletsInputTotal
            );
        
        return $json;
    }
    
    public function getListPalletsDetails($tokenForm)
    {
        $repoManInputs = new ManufacturingInputTempRepository();
        $pallets = $repoManInputs->getPalletList($tokenForm);
        
        $details = $this->listPalletsDetails($pallets);
        $json = array(
                'response' => true,
                'palletsDetails' => $details['palletsDetails'],
                'distinctLots'=>$details['distinctLots'],
                'cajasTotal'=>$details['cajasTotal'],
                'palletsTotal'=>$details['palletsTotal'],
                'pesoTotal'=>$details['pesoTotal'],
                'cajasInputTotal'=>$details['cajasInputTotal'],
                'palletsInputTotal'=>$details['palletsInputTotal']
            );
        return $json;
    }
    
    public function setInputPallet($options)
    {
        $idDetail = $options['idDetail'];
        $repo = new ManufacturingInputTempRepository();
       
        if (!$repo->setInputPallet($idDetail)) {
            return array(
                'response'=>false,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
       
        $json = $this->getListPalletsDetails($options['token_form']);
        $detailsByLot = '';
        
        if ($options['activeLot']!= '') {
            $pallets = $repo->getInputsByReference($options['activeLot'], $options['token_form']);
            $details = $this->listPalletsDetails($pallets);
            $detailsByLot = $details['palletsDetails'];
        }
        
        $json['detailsByLot'] = $detailsByLot;
        $json['inputSummary'] = $this->getSummaryInputs($options['token_form']);
        return $json;
    }
    
    public function editInputBoxes($options)
    {
        $repo = new ManufacturingInputTempRepository();
        $rs = $repo->editInputBoxes($options);
        
        if ($rs == null) {
            return array(
                'response'=>false,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
        
        $json = $this->getListPalletsDetails($options['token_form']);
        $detailsByLot = '';
        
        if ($options['activeLot']!= '') {
            $pallets = $repo->getInputsByReference($options['activeLot'], $options['token_form']);
            $details = $this->listPalletsDetails($pallets);
            $detailsByLot = $details['palletsDetails'];
        }
        
        $json['detailsByLot'] = $detailsByLot;
        $json['inputSummary'] = $this->getSummaryInputs($options['token_form']);
        return $json;
    }
    
    #Proceso en Modal
    #Setea en Modal informacion de Lote seleccionado
    public function setMfgDetails($options)
    {
        $lot = $options['lot'];
        
        $repo = new ManufacturingInputTempRepository();
        $pallets = $repo->getInputsByReference($lot, $options['token_form']);
        $details = $this->listPalletsDetails($pallets);
        
        $repo = new ManufacturingOutputTempRepository();
        $outputBoxes = $repo->getOutputsByReferenceAndLot($lot, $options['token_form']);
        $listOutputBoxes = $this->listOutputBoxes($outputBoxes);
        
        $repo = new ManufacturingWasteTempRepository();
        $wasteBoxes = $repo->getWasteByReferenceAndLot($lot, $options['token_form']);
        $listWasteBoxes = $this->listWasteBoxes($wasteBoxes);
        
        $json = array(
                'response' => true,
                'inputDetails' => $details['palletsDetails'],
                'outputDetails' => $listOutputBoxes['outputDetails'],
                'wasteDetails' => $listWasteBoxes['wasteDetails'],
                'distinctLots'=>$details['distinctLots'],
                'cajasTotal'=>$details['cajasTotal'],
                'palletsTotal'=>$details['palletsTotal'],
                'pesoTotal'=>$details['pesoTotal'],
                'cajasInputTotal'=>$details['cajasInputTotal'],
                'palletsInputTotal'=>$details['palletsInputTotal']
            );

        return $json;
    }
    
    #Agregar Salidas
    public function setOutputBoxes($options)
    {
        $idDetail = $options['idDetail'];
        $data = array();
        foreach ($options['options'] as $row) {
            $data[$row['name']] = $row['value'];
        }
        
        #Obtenemos  id_receiving y id_manifest de un input que pertenezca a $data['lot'];
        $repoMFGInput = new ManufacturingInputTempRepository();
        $dataInput = $repoMFGInput->getDataByReference($data['lot'], $data['token_form']);
        
        $data['id_receiving'] = $dataInput['id_receiving'];
        $data['id_manifest'] = $dataInput['id_manifest'];
        //$data['reference'] = $dataInput['type']."|".$dataInput['reference'];
        
        $repo = new ManufacturingOutputTempRepository();
        $repo->setOptions($data);
        $data = $repo->getOptions();
        
        if ($idDetail === '' || $idDetail === null) {
            $repo->save($data);
        } else {
            $repo->update($idDetail, $data);
        }
        
        $outputBoxes = $repo->getOutputsByReferenceAndLot($data['lot'], $data['token_form']);
        $listOutputBoxes = $this->listOutputBoxes($outputBoxes);
        
        return array(
            'response'=>true,
            'outputDetails'=>$listOutputBoxes['outputDetails'],
            'outputSummary'=>$this->getSummaryOutputs($data['token_form']),
            'outputSummaryByLot'=>$this->getSummaryOutputsByLot($data['token_form'])
        );
    }
    
    public function isEditableOB($options)
    {
        $repo = new ManufacturingOutputTempRepository();
        $boxesInUse = $repo->getOutputBoxesInPalletsByIdDetail($options['id'], $options['token_form']);
        
        if ($boxesInUse > 0) {
            $this->flashmessenger->addMessage(array('danger'=>'Este producto no puede ser editado o eliminado, se esta utilizando en Salida Pallets.'));
            return array(
                'response'=>false,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
        
        return array(
            'response'=>true
        );
    }
    
    #Elimina Salidas
    public function deleteOutputBoxes($options)
    {
        $repo = new ManufacturingOutputTempRepository();
        $boxesInUse = $repo->getOutputBoxesInPalletsByIdDetail($options['id'], $options['token_form']);
        
        if ($boxesInUse > 0) {
            $this->flashmessenger->addMessage(array('danger'=>'Este producto no puede ser eliminado, se esta utilizando en Salida Pallets.'));
            return array(
                'response'=>false,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
        
        $repo->deleteByOutputDetail($options['id'], $options['token_form']);
        
        $outputBoxes = $repo->getOutputsByReference($options['activeLot'], $options['token_form']);
        $listOutputBoxes = $this->listOutputBoxes($outputBoxes);
        
        return array(
            'response'=>true,
            'outputDetails'=>$listOutputBoxes['outputDetails'],
            'outputSummary'=>$this->getSummaryOutputs($options['token_form']),
            'outputSummaryByLot'=>$this->getSummaryOutputsByLot($options['token_form'])
        );
    }
    
    #Lista Salidas
    public function listOutputBoxes($outputBoxes)
    {
        $login = new Login();
        $string = "";
        if ($outputBoxes !== null && count($outputBoxes) > 0) {
            foreach ($outputBoxes as $item) {
                $id = $item['id'];
                $array = json_encode($item);
                $string .= "<tr> "
                            .  "<td class='text-center'>{$item['descripcion']}</td>"
                            .  "<td class='text-center'>{$item['varietyName']}</td>"
                            .  "<td class='text-center'>{$item['sizeName']}</td>"
                            .  "<td class='text-center'>{$item['packingName']}</td>"
                            .  "<td class='text-center'>{$item['qualityName']}</td>"
                            .  "<td class='text-center'>{$item['colorName']}</td>"
                            .  "<td class='text-right'>{$item['weight']}</td>"
                            .  "<td class='text-right'>{$item['quantity_boxes']}</td>"
                            .  "<td class='text-center'>";
                            
                if ($login->getRole() != '3') {
                    $string .= "<a class='btn btn-xs btn-default edit' data-data='$array' data-mixto='0' data-controller='Manufacturing'><i class='fa fa-pencil'></i></a>
                                <a class='btn btn-xs btn-danger' onclick='deleteOutputBoxes(\"{$item['id_output_detail']}\");'><i class='fa fa-trash'></i></a>";
                }
                   
                $string .=     "</td>";
                $string .=  "</tr>";
            }
        }
        
        return array(
            'outputDetails'=>$string
        );
    }
    
    #Agregar Merma
    public function setWasteBoxes($options)
    {
        $idDetail = $options['idDetail'];
        $data = array();
        foreach ($options['options'] as $row) {
            $data[$row['name']] = $row['value'];
        }
        
        /*
        #Obtenemos  id_receiving y id_manifest de un input que pertenezca a $data['lot'];
        $repoMFGInput = new ManufacturingInputTempRepository();
        $dataInput = $repoMFGInput->getDataByReference($data['reference'],$data['token_form']);
        //$data['id_receiving'] = $dataInput['id_receiving'];
        //$data['id_manifest'] = $dataInput['id_manifest'];
        $data['lot'] = $dataInput['lot'];
        $data['reference'] = $dataInput['type']."|".$dataInput['reference'];
        */
        $repo = new ManufacturingWasteTempRepository();
        $repo->setOptions($data);
        $data = $repo->getOptions();
        
        if ($idDetail === '' || $idDetail === null) {
            $repo->save($data);
        } else {
            $repo->update($idDetail, $data);
        }
        
        $outputBoxes = $repo->getWasteByReferenceAndLot($data['lot'], $data['token_form']);
        $listOutputBoxes = $this->listWasteBoxes($outputBoxes);
        
        return array(
            'response'=>true,
            'wasteDetails'=>$listOutputBoxes['wasteDetails'],
            'wasteSummary'=>$this->getSummaryWaste($data['token_form'])
        );
    }
    
    #Elimina Merma
    public function deleteWasteBoxes($options)
    {
        $repo = new ManufacturingWasteTempRepository();
        $repo->delete($options['id']);
        
        $outputBoxes = $repo->getWasteByReferenceAndLot($options['activeLot'], $options['token_form']);
        $listOutputBoxes = $this->listWasteBoxes($outputBoxes);
        
        return array(
            'response'=>true,
            'wasteDetails'=>$listOutputBoxes['wasteDetails'],
            'wasteSummary'=>$this->getSummaryWaste($options['token_form'])
        );
    }
    
    #Lista Mermas
    public function listWasteBoxes($outputBoxes)
    {
        $login = new Login();
        $string = "";
        if ($outputBoxes !== null && count($outputBoxes) > 0) {
            foreach ($outputBoxes as $item) {
                $id = $item['id'];
                $array = json_encode($item);
                $string .= "<tr> "
                            .  "<td class='text-center'>{$item['descripcion']}</td>"
                            .  "<td class='text-center'>{$item['varietyName']}</td>"
                            .  "<td class='text-center'>{$item['sizeName']}</td>"
                            .  "<td class='text-center'>{$item['packingName']}</td>"
                            .  "<td class='text-center'>{$item['qualityName']}</td>"
                            .  "<td class='text-center'>{$item['colorName']}</td>"
                            .  "<td class='text-right'>{$item['weight']}</td>"
                            .  "<td class='text-right'>{$item['quantity_boxes']}</td>";
                            
                $string .="<td class='text-center'>";
                if ($login->getRole() != '3') {
                    $string .= "<a class='btn btn-xs btn-default edit' data-data='$array' data-mixto='0' data-controller='Manufacturing'><i class='fa fa-pencil'></i></a>
                                <a class='btn btn-xs btn-danger' onclick='deleteWasteBoxes($id);'><i class='fa fa-trash'></i></a>";
                }
                            
                $string .=     "</td>";
                $string .=  "</tr>";
            }
        }
        
        return array(
            'wasteDetails'=>$string
        );
    }
    
    #Lista Distintos Lotes
    public function setDistinctLots($tokenForm)
    {
        $repoManInputs = new ManufacturingInputTempRepository();
        $lots = $repoManInputs->getDistinctReferenceLots($tokenForm);
        $distinctLots = null;
        
        #Antes manejaba lot, se actualizo para usar Type|Reference (Ejem: Maniefest|2 o Manufacturin|4)
        if ($lots) {
            foreach ($lots as $lot) {
                $distinctLots .= "<span class='btn btn-md btn-default m-r-1' data-lot='{$lot['lot']}' onclick='setMfgDetails(this)'>"
                                    .$this->_getTranslation('Lote:'). " {$lot['lot']}<br/>"
                               . "</span>";
            }
        }
        
        return $distinctLots;
    }
    
    #Muestra tabla inputSummary con listado de todas las cajas Input sin importar pallet ni lote.
    public function getSummaryInputs($tokenForm)
    {
        $repoInput = new ManufacturingInputTempRepository();
        $inputSummary = $repoInput->getSummaryBoxes($tokenForm);
        $inputString = '';
        
        if (is_array($inputSummary) && count($inputSummary) > 0) {
            foreach ($inputSummary as $item) {
                $inputString .= "<tr> "
                            .  "<td class='text-center'>{$item['descripcion']}</td>"
                            .  "<td class='text-center'>{$item['varietyName']}</td>"
                            .  "<td class='text-center'>{$item['sizeName']}</td>"
                            .  "<td class='text-center'>{$item['packingName']}</td>"
                            .  "<td class='text-center'>{$item['qualityName']}</td>"
                            .  "<td class='text-center'>{$item['colorName']}</td>"
                            .  "<td class='text-right'>{$item['weight']}</td>"
                            .  "<td class='text-right'>{$item['quantity_boxes']}</td>"
                            ."</tr>";
            }
        }
        
        return $inputString;
    }
    
    #Muestra tabla outputSummary con listado de todas las cajas Output sin importar pallet ni lote.
    public function getSummaryOutputs($tokenForm)
    {
        $repoOutput = new ManufacturingOutputTempRepository();
        $outputSummary = $repoOutput->getSummaryBoxes($tokenForm);
        $outputString = '';
        //var_dump($outputSummary);
        if (is_array($outputSummary) && count($outputSummary) > 0) {
            foreach ($outputSummary as $item) {
                $outputString .= "<tr> "
                            .  "<td class='text-center'>{$item['descripcion']}</td>"
                            .  "<td class='text-center'>{$item['varietyName']}</td>"
                            .  "<td class='text-center'>{$item['sizeName']}</td>"
                            .  "<td class='text-center'>{$item['packingName']}</td>"
                            .  "<td class='text-center'>{$item['qualityName']}</td>"
                            .  "<td class='text-center'>{$item['colorName']}</td>"
                            .  "<td class='text-right'>{$item['weight']}</td>"
                            .  "<td class='text-right'>{$item['quantity_boxes']}</td>"
                            ."</tr>";
            }
        }
        
        return $outputString;
    }
    
    #Muestra tabla wasteSummary con listado de todas las cajas Waste sin importar pallet ni lote.
    public function getSummaryWaste($tokenForm)
    {
        $repoWaste = new ManufacturingWasteTempRepository();
        $wasteSummary = $repoWaste->getSummaryBoxes($tokenForm);
        $wasteString = '';
        
        if (is_array($wasteSummary) && count($wasteSummary) > 0) {
            foreach ($wasteSummary as $item) {
                $wasteString .= "<tr> "
                            .  "<td class='text-center'>{$item['descripcion']}</td>"
                            .  "<td class='text-center'>{$item['varietyName']}</td>"
                            .  "<td class='text-center'>{$item['sizeName']}</td>"
                            .  "<td class='text-center'>{$item['packingName']}</td>"
                            .  "<td class='text-center'>{$item['qualityName']}</td>"
                            .  "<td class='text-center'>{$item['colorName']}</td>"
                            .  "<td class='text-right'>{$item['weight']}</td>"
                            .  "<td class='text-right'>{$item['quantity_boxes']}</td>"
                            ."</tr>";
            }
        }
        
        return $wasteString;
    }
    
    #Muestra tablas outputSummary con listado de todas las cajas Output sin importar pallet ni lote.
    public function getSummaryOutputsByLot($tokenForm)
    {
        $login = new Login();
        $classDraggable = 'draggable_tr';
        if ($login->getRole() == '3') {
            $classDraggable = '';
        }
        
        $repoOutput = new ManufacturingOutputTempRepository();
        $outputSummary = $repoOutput->getSummaryBoxesByLot($tokenForm);
        $outputString = '';
        
        if (is_array($outputSummary) && count($outputSummary) > 0) {
            foreach ($outputSummary as $item) {
                $outputString .= "<tr class='$classDraggable' data-iddetail_ob='{$item['id_output_detail']}'> "
                            .  "<td class='text-center'>{$item['descripcion']}</td>"
                            .  "<td class='text-center'>{$item['varietyName']}</td>"
                            .  "<td class='text-center'>{$item['sizeName']}</td>"
                            .  "<td class='text-center'>{$item['packingName']}</td>"
                            .  "<td class='text-center'>{$item['qualityName']}</td>"
                            .  "<td class='text-center'>{$item['colorName']}</td>"
                            .  "<td class='text-center'>{$item['weight']}</td>"
                            .  "<td class='text-right qtyBoxes'>{$item['quantity_boxes']}</td>"
                            .  "<td class='text-center'>{$item['lot']}</td>"
                            ."</tr>";
            }
        }
        
        return $outputString;
    }
    
    /* FOR  OUTPUT PALLETS */
    
    public function addTicketToSplit($options)
    {
        $i = $options['currentNumTicket'];
        $i++;
        $emptyTicket = "<div class='col-lg-6 col-md-6 col-sm-12'>
                            <h5>".$this->_getTranslation('Pallet')." "."_$i</h5>
                                <table id='pallet-$i' class='childgrid pallet table table-condensed table-bordered font-size-11'>
                                    <thead>
                                        <th class='text-center'>".$this->_getTranslation('Descripcion')."  </th>
                                        <th class='text-center'>".$this->_getTranslation('Variedad')."  </th>
                                        <th class='text-center'>".$this->_getTranslation('Tamaño')."  </th>
                                        <th class='text-center'>".$this->_getTranslation('Empaque')."  </th>
                                        <th class='text-center'>".$this->_getTranslation('Calidad')."  </th>
                                        <th class='text-center'>".$this->_getTranslation('Color')."  </th>
                                        <th class='text-center'>".$this->_getTranslation('Peso')."  </th>
                                        <th class='text-right'>".$this->_getTranslation('Cajas')."</th>
                                        <th class='text-center'>".$this->_getTranslation('Lote')."</th>                                        
                                    </thead>
                                    <tbody></tbody>
                                </table>
                            </div>";
        
        return array(
            'response'=>true,
            'newTicket'=>$emptyTicket,
            'currentNumTicketSplit'=> $i
        );
    }
    
    public function setProductInPallet($options)
    {
        $repoOutput = new ManufacturingOutputTempRepository();
        $data = $repoOutput->setPalletOutput($options);
        
        if ($data['response'] === true) {
            return array(
                'response'=>true,
                'info'=>json_encode($data['info'])
            );
        }
        
        return array(
            'response'=>false,
            'msg'=>$data['msg']
        );
    }
    
    public function moveProductInPallet($options)
    {
        $repoOutput = new ManufacturingOutputTempRepository();
        $data = $repoOutput->moveProductInPallet($options);
        
        if ($data['response'] === true) {
            return array(
                'response'=>true,
                'info'=>json_encode($data['info'])
            );
        }
        
        return $data;
    }
    
    public function deleteProductInPallet($options)
    {
        $repoOutput = new ManufacturingOutputTempRepository();
        $data = $repoOutput->deleteProductInPallet($options);
        
        if ($data['response'] === true) {
            return array(
                'response'=>true
            );
        }
        
        return array(
                'response'=>false,
                'msg'=>$data['msg']
            );
    }
    
    #Muestra tablas outputSummary con listado de todas las cajas Output sin importar pallet ni lote.
    public function getOutputsPallets($tokenForm)
    {
        $login = new Login();
        $repoOutput = new ManufacturingOutputTempRepository();
        $outputPallets = $repoOutput->getOutputPallets($tokenForm);
        $outputPalletsString = '';
        
        $classChildGrid = 'childgrid';
        $classDraggable = 'draggable_tr';
        
        if ($login->getRole() == '3') {
            $classChildGrid = '';
            $classDraggable = '';
        }
        
        if (is_array($outputPallets) && count($outputPallets) > 0) {
            foreach ($outputPallets as $consecutivepallet => $palletDetails) {
                $outputPalletsString .= "<div class='col-lg-6 col-md-6 col-sm-12'>
                                        <h5>".$this->_getTranslation('Pallet')." "."$consecutivepallet</h5>
                                            <table id='pallet-$consecutivepallet' class='$classChildGrid pallet table table-condensed table-bordered font-size-11'>
                                                <thead>
                                                    <th class='text-center'>".$this->_getTranslation('Descripcion')."  </th>
                                                    <th class='text-center'>".$this->_getTranslation('Variedad')."  </th>
                                                    <th class='text-center'>".$this->_getTranslation('Tamaño')."  </th>
                                                    <th class='text-center'>".$this->_getTranslation('Empaque')."  </th>
                                                    <th class='text-center'>".$this->_getTranslation('Calidad')."  </th>
                                                    <th class='text-center'>".$this->_getTranslation('Color')."  </th>
                                                    <th class='text-center'>".$this->_getTranslation('Peso')."  </th>
                                                    <th class='text-right'>".$this->_getTranslation('Cajas')."</th>
                                                    <th class='text-center'>".$this->_getTranslation('Lote')."</th>                                        
                                                </thead>
                                                <tbody>";
                
                foreach ($palletDetails as $detail) {
                    $outputPalletsString .= "<tr class='$classDraggable' data-iddetail_ob='{$detail['id_output_detail']}' data-iddetail_op='{$detail['id']}' data-quantity='{$detail['quantity_boxes']}'> "
                            .  "<td class='text-center'>{$detail['descripcion']}</td>"
                            .  "<td class='text-center'>{$detail['varietyName']}</td>"
                            .  "<td class='text-center'>{$detail['sizeName']}</td>"
                            .  "<td class='text-center'>{$detail['packingName']}</td>"
                            .  "<td class='text-center'>{$detail['qualityName']}</td>"
                            .  "<td class='text-center'>{$detail['colorName']}</td>"
                            .  "<td class='text-center'>{$detail['weight']}</td>"
                            .  "<td class='text-right qtyBoxes'>{$detail['quantity_boxes']}</td>"
                            .  "<td class='text-center'>{$detail['lot']}</td>"
                            ."</tr>";
                }
                $outputPalletsString .= "</tbody></table></div>";
            }
        }
        
        return array(
            'consecutive_pallet'=>$repoOutput->getCurrentConsecutivoPallet($tokenForm),
            'outputPallets'=>$outputPalletsString
        );
    }
    
    /* END FOR OUTPUT PALLETS*/
    
    public function clearInfoLoteFromMFG($options)
    {
        $result = $this->clearInfoLote($options['lot'], $options['token_form']);
        if ($result['response']===true) {
            $r1 = $this->getListPalletsDetails($options['token_form']);
            
            $repository = new ManufacturingOutputTempRepository();
            $shippingRequestDetalles = $repository->getOuputPalletDetails($options['token_form']);
            $detalles = $this->listOutputPalletsDetalles($shippingRequestDetalles);
            
            return array(
                'response'=>true,
                'palletsDetails'=>$r1['palletsDetails'],
                'cajasTotal'=>$r1['cajasTotal'],
                'palletsTotal'=>$r1['palletsTotal'],
                'pesoTotal'=>$r1['pesoTotal'],
                'cajasInputTotal'=>$r1['cajasInputTotal'],
                'palletsInputTotal'=>$r1['palletsInputTotal'],
                
                'distinctLots'=>$this->setDistinctLots($options['token_form']),
                'inputSummary'=>$this->getSummaryInputs($options['token_form']),
                'outputSummary'=>$this->getSummaryOutputs($options['token_form']),
                'wasteSummary'=>$this->getSummaryWaste($options['token_form']),
                'outputSummaryByLot'=>$this->getSummaryOutputsByLot($options['token_form']),
                'outputPallets'=>$detalles['listDetalles']
                
            );
        }
        return $result;
    }
    
    public function allowEditManufacturing($options)
    {
        $entityRepository = new EntityRepository();
        $entityRepository->update($options['idManufacturing'], array('allow_edit'=>'1'), 'manufacturing');
        return array(
            'response'=>true,
        );
    }
    
    /* NEW MANUFACTURING*/
    
    public function getDataPalletById($options)
    {
        $tokenForm = $options['token_form'];
        $palletId = $options['pallet_id'];
        $listado = null;
        $activado = null;

        $inventarioRepo = new InventoryRepository();
        $palletData = $inventarioRepo->getByPalletId($palletId);
        
        if ($palletData) {
            $reference = $palletData[0]['lot'];
            
            /*Validar si ya existe pallet listado inputs*/
            $inputRepo = new ManufacturingInputTempRepository();
            $existPalletInInput = $inputRepo->isListedByReference($reference, $tokenForm);
            
            if ($existPalletInInput) {
                $listado = true;
                if ($inputRepo->isActivedPalletId($palletId, $tokenForm)) {
                    $activado = true;
                    $this->flashmessenger->addMessage(array('info'=>"El pallet {%$palletId%} ya esta ingresado."));
                }
            }
            
            if ($palletData[0]['quantity_boxes'] <= 0) {
                $this->flashmessenger->addMessage(array('info'=>"El pallet {%$palletId%} no tiene stock."));
                return array(
                    'response'=>null,
                    'msg'=>$this->flashmessenger->getRawMessage()
                );
            }

            return array(
                'response'=>true,
                'lot'=>$reference,
                'listed'=>$listado,
                'actived'=>$activado,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
        
        $this->flashmessenger->addMessage(array('info'=>"El pallet {%$palletId%} no existe en el sistema."));
        return array(
            'response'=>null,
            'msg'=>$this->flashmessenger->getRawMessage()
        );
    }
    
    public function getAvailableInventoryByMfgOutput($options)
    {
        $idProduct = $options['id_product'];
        $tokenForm = $options['token_form'];
        
        $repoProduct = new ProductRepository();
        $dataProduct = $repoProduct->getById($idProduct);
        $message = "<h4>".$this->_getTranslation('Stock disponible para')." ".$dataProduct['descripcion']."</h4>";
        
        $repoInventory = new ManufacturingOutputTempRepository();
        $availableInventory = $repoInventory->getSummaryInventoryByMfgOutput($idProduct, $tokenForm);
        
        $string = '';
        foreach ($availableInventory as $producto) {
            $infoProducto = json_encode($producto);
            
            #setReceivingDetail funcion generica para popular formularios con info enviada. nombrecampo=>valor
            $string .= "<tr style='cursor:pointer' onclick='setInfoProductOuputPallet($infoProducto)'>";
     
            $string .=  "   <td style='text-align:center'>{$producto['lot']}</td>
                            <td style='text-align:center'>{$producto['descripcion']}</td>
                            <td style='text-align:center'>{$producto['varietyName']}</td>
                            <td style='text-align:center'>{$producto['madurityName']}</td>
                            <td style='text-align:center'>{$producto['sizeName']}</td>
                            <td style='text-align:center'>{$producto['packingName']}</td>
                            <td style='text-align:center'>{$producto['brandName']}</td>
                            <td style='text-align:center'>{$producto['qualityName']}</td>                                
                            <td style='text-align:center'>{$producto['colorName']}</td>
                            <td style='text-align:center'>{$producto['weight']}</td>
                            <td style='text-align:center'>{$producto['quantity_boxes']}</td>
                            <td style='text-align:center' class='success'>{$producto['available']}</td>
                       </tr>";
        }
        
        $json = array(
                 'response'=>true,
                 'availableInventory'=>$string,
                 'message'=>$message);
        return $json;
    }
    
    public function setOutputPalletsDetalles(array $options)
    {
        $msg = null;
        $data = array();
        foreach ($options['options'] as $row) {
            $data[$row['name']] = $row['value'];
        }
        
        $repo = new ManufacturingOutputTempRepository();
        $producto = $repo->getProductByIdOutputDetail($data['id_output_detail']);
        $data['descripcion'] = $producto['descripcion'];
        $data['id_receiving'] = $producto['id_receiving'];
        $data['id_manifest'] = $producto['id_manifest'];
        $data['reference'] = $producto['reference'];

        if (!$repo->insertOutputPalletDetalle($data)) {
            return array(
                'response'=>false,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
            
        $ouputPalletsDetails = $repo->getOuputPalletDetails($data['token_form']);
        $detalles = $this->listOutputPalletsDetalles($ouputPalletsDetails);
        $ouputPalletsDetailsMixto = null;
        $detallesMixto = null;

        if ($data['mixto'] !== '0' && $data['mixto'] !== null) {
            $ouputPalletsDetailsMixto = $repo->getOuputPalletDetailsByMixto($data['mixto'], $data['token_form']);
            $detallesMixto = $this->listOuputPalletDetailsMixto($ouputPalletsDetailsMixto);
        }

        $json = array(
            'response' => true,
            'outputPalletsDetails' => $detalles['listDetalles'],
            'cajasTotal'=>$detalles['cajasTotal'],
            'palletsTotal'=>$detalles['palletsTotal'],
            'pesoTotal'=>$detalles['pesoTotal'],
            'outputPalletsDetailsMixto' => $detallesMixto['listDetalles'],
            'cajasTotalMixto' => $detallesMixto['cajasTotal'],
            'pesoTotalMixto' => $detallesMixto['pesoTotal'],
        );
        
        if ($msg) {
            $json['msg'] = $this->flashmessenger->getMessageString();
        }

        return $json;
    }
    
    public function listOutputPalletsDetalles($detalles)
    {
        $listDetalles = "";
        $cajasTotal = 0;
        $palletsTotal = 0;
        $pesoTotal = 0;
        $sumaPesoTotal = 0;
        $tokenForm = null;
        
        foreach ($detalles as $detalle) {
            $tokenForm = $detalle['token_form'];
            $detalle['quantity_boxes_per_pallets'] = $detalle['quantity_boxes'];
            $array = json_encode($detalle);
            $cajasTotal += $detalle['quantity_boxes'];
            $palletsTotal += $detalle['quantity_pallets'];
            $sumaPesoTotal += $detalle['weight_total'];
            
            $descripcion = str_replace(',', "<br>", $detalle['descripcion']);
            $variety = str_replace(',', "<br>", $detalle['varietyName']);
            $size = str_replace(',', "<br>", $detalle['sizeName']);
            $packing = str_replace(',', "<br>", $detalle['packingName']);
            $quality = str_replace(',', "<br>", $detalle['qualityName']);
            $color = str_replace(',', "<br>", $detalle['colorName']);
            $weight = str_replace(',', "<br>", $detalle['weight_concat']);
            $boxesQuantity = str_replace(',', "<br>", $detalle['quantity_boxes_concat']);
            $palletsQuantity = str_replace(',', "<br>", $detalle['quantity_pallets_concat']);
            $lot = str_replace(',', "<br>", $detalle['lot_concat']);
            
            $id = $detalle['id'];
            $listDetalles .= "<tr>"
                . "<td class='text-center' style='white-space:nowrap'>
                    <input type='checkbox' class='_checkboxPalletId btn' data-id='$id' data-mixto='{$detalle['mixto']}' style='width:27px;height:27px;margin-top:0px' />
                    <a class='btn btn-sm btn-default' onclick='setDetalleOutputPallesDetailsToEdit($array,this);' data-mixto='{$detalle['mixto']}' data-controller='Manufacturing_output'><i class='fa fa-pencil'></i></a>
                    <a class='btn btn-sm btn-danger' onclick='deleteOutputPallesDetails($id,this);' data-mixto='{$detalle['mixto']}'><i class='fa fa-trash'></i></a>
                </td>  "
                ."<td class='text-center'>".$detalle['id_pallet']."</td>"
                ."<td class='text-center'>".$descripcion."</td>"
                ."<td class='text-center'>".$variety."</td>"
                ."<td class='text-center'>".$size."</td>"
                ."<td class='text-center'>".$packing."</td>"
                ."<td class='text-center'>".$quality."</td>"
                ."<td class='text-center'>".$color."</td>"
                ."<td class='text-right'>".$weight."</td>"
                ."<td class='text-right'>".$boxesQuantity."</td>"
                ."<td class='text-center'>".$lot."</td>"
                ."</tr>";
        }
        
        return array('listDetalles'=>$listDetalles,
                     //'listSummary'=>$this->listShippingRequestSummary($tokenForm),
                     'cajasTotal'=>$cajasTotal,
                     'palletsTotal'=>$palletsTotal,
                     'pesoTotal'=>$sumaPesoTotal);
    }
    
    public function listOuputPalletDetailsMixto($detalles)
    {
        $listDetalles = "";
        $cajasTotal = 0;
        $pesoTotal = 0;
        $sumaPesoTotal = 0;
        
        foreach ($detalles as $detalle) {
            $detalle['quantity_boxes_per_pallets'] = $detalle['quantity_boxes'];
            $array = json_encode($detalle);
            $cajasTotal += $detalle['quantity_boxes'];
            $sumaPesoTotal += $detalle['weight'];
            $id = $detalle['id'];
            $listDetalles .= "<tr>"
                . "<td class='text-left'>                       
                    <a class='btn btn-xs btn-default' onclick='setDetalleOuputPalletsToEditMixto($array);'><i class='fa fa-pencil'></i></a>
                    <a class='btn btn-xs btn-danger' onclick='deleteOutputPallesDetailsMixto($id);'><i class='fa fa-trash'></i></a>
                </td>  "
               ."<td class='text-center'>".$detalle['descripcion']."</td>"
               ."<td class='text-center'>".$detalle['varietyName']."</td>"
               ."<td class='text-center'>".$detalle['sizeName']."</td>"
               ."<td class='text-center'>".$detalle['qualityName']."</td>"
               ."<td class='text-right'>".$detalle['quantity_boxes']."</td>"
               ."</tr>";
        }
        
        return array('listDetalles'=>$listDetalles,
                     'cajasTotal'=>$cajasTotal,
                     'pesoTotal'=>$sumaPesoTotal);
    }
    
    public function getListOutputPalletsDetalles($tokenForm)
    {
        $repo = new ManufacturingOutputTempRepository();
        $shippingDetalles = $repo->getOuputPalletDetails($tokenForm);
        $detalles = $this->listOutputPalletsDetalles($shippingDetalles);

        $json = array(
                'response' => true,
                'outputPalletsDetails' => $detalles['listDetalles'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'pesoTotal'=>$detalles['pesoTotal']
            );
        return $json;
    }
    
    public function getListOuputPalletDetailsMixto($options)
    {
        $repo = new ManufacturingOutputTempRepository();
        $ouputPalletsDetailsMixto = $repo->getOuputPalletDetailsByMixto($options['mixto'], $options['token_form']);
        $detallesMixto = $this->listOuputPalletDetailsMixto($ouputPalletsDetailsMixto);
        
        $json = array(
            'response' => true,
            'outputPalletsDetailsMixto' => $detallesMixto['listDetalles'],
            'cajasTotalMixto' => $detallesMixto['cajasTotal'],
            'pesoTotalMixto' => $detallesMixto['pesoTotal']
        );
        
        return $json;
    }
    
    public function deleteDetalles(array $options)
    {
        $id = $options['id'];
        $repository = new ManufacturingOutputTempRepository();
        $data = $repository->getByIdOutputPallet($id);
        $tokenForm = $data['token_form'];
        
        if ($data['mixto']!=='0') {
            $result = $repository->deleteDetalleByMixto($data['mixto'], $tokenForm);
        } else {
            $result = $repository->deleteOutputPallet($id);
        }
        
        if ($result) {
            $response = true;
            $msj = 'Producto eliminado correctamentes.';
        } else {
            $response = null;
            $msj = $this->flashmessenger->getRawMessage();
        }
        
        $shippingRequestDetalles = $repository->getOuputPalletDetails($tokenForm);
        $detalles = $this->listOutputPalletsDetalles($shippingRequestDetalles);
        
        $json = array(
                'response' => $response,
                'message'=>$this->_getTranslation($msj),
                'outputPalletsDetails'=>$detalles['listDetalles'],
                'pesoTotal'=>$detalles['pesoTotal'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
            );
        return $json;
    }
    
    public function deleteOutputPallesDetailsMixto(array $options)
    {
        $id = $options['id'];
        $repository = new ManufacturingOutputTempRepository();
        $currentData = $repository->getByIdOutputPallet($id);
        //var_dump($currentData);
        if ($repository->deleteOutputPallet($id)) {
            $repository->assignMainProductForMixto($currentData['mixto'], $currentData['token_form']);
            $response = true;
            $msj = 'Producto eliminado correctamentes.';
        } else {
            $response = null;
            $msj = "No se pudo eliminar producto. Intente nuevamente.";
        }
        
        $ouputPalletsDetails = $repository->getOuputPalletDetails($currentData['token_form']);
        $detalles = $this->listOutputPalletsDetalles($ouputPalletsDetails);
        
        $dataMixto = $this->getListOuputPalletDetailsMixto(array('mixto'=>$currentData['mixto'],'token_form'=>$currentData['token_form']));
        $json = array(
                'response' => $response,
                'message'=>$this->_getTranslation($msj),
                'outputPalletsDetails'=>$detalles['listDetalles'],
                'pesoTotal'=>$detalles['pesoTotal'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'outputPalletsDetailsMixto' => $dataMixto['outputPalletsDetailsMixto'],
                'cajasTotalMixto' => $dataMixto['cajasTotalMixto'],
                'pesoTotalMixto' => $dataMixto['pesoTotalMixto'],
            );
        return $json;
    }
}
