<?php
class PurchaseGoodAndServiceAjax extends PurchaseGoodAndServiceRepository
{
    public $formProduct = null;
    public $formPurchase = null;
    public $formVendor = null;
    
    public function __construct()
    {
        parent::__construct();
    }

    public function getResponse($request, $options)
    {
        return $this->$request($options);
    }

    public function setPurchaseDetails(array $options)
    {
        $data = array();
        foreach ($options['options'] as $row) {
            $data[$row['name']] = $row['value'];
        }
        
        $producto = $this->getProductById($data['id_product']);
        if ($producto) {
            $data['description'] = $producto['description'];
            
            $this->insertDetalle($data);
            $compraDetalles = $this->getPurchaseDetails($data['token_form']);
            $detalles = $this->listCompraDetalles($compraDetalles);

            $json = array(
                'response' => true,
                'purchaseDetails' => $detalles['listDetails'],
                'totalItems'=>number_format($detalles['totalItems'], 2),
                'total_importe'=>$detalles['total_importe'],
                'descuento_items'=>$detalles['descuento_items'],
                'descuento_general'=>$detalles['descuento_general'],
                'total_descuentos'=>number_format($detalles['total_descuentos'], 2),
                'total_subtotal'=>number_format($detalles['total_subtotal'], 2),
                'total_impuestos'=>$detalles['total_impuestos'],
                'total'=>$detalles['total']
            );
            
            return $json;
        } else {
            $this->flashmessenger->addMessage(array('danger'=>$this->_getTranslation('Producto no registrado.')));
            return $json = array(
                'response'=>null,
                'message'=>$this->flashmessenger->getMessageString());
        }
    }
    
    public function listCompraDetalles($detalles)
    {
        $listDetalles = "";
        $cantidadItems = 0;
        $total_importe = 0;
        $descuentos_item = 0;
        $descuentos_general = 0;
        $total_descuentos = 0;
        $total_subtotal = 0;
        $total_impuestos = 0;
        $total_impuestos_string = '';
        $taxes_applied = array();
        $total = 0;
        
        foreach ($detalles as $detalle) {
            $id = $detalle['id'];
            $discountGeneral = $detalle['discount_general'];
            
            unset($detalle['id'],$detalle['discount_general'],$detalle['discount_general_type']);
            $detalle['product'] = $detalle['description']." (".$detalle['code'].")";
            $array = json_encode($detalle);
            $cantidadItems += $detalle['quantity'];
            $total_importe += $detalle['cost'] * $detalle['quantity'];
            $descuentos_item += $detalle['discount_amount'];
            $descuentos_general += $detalle['discount_general_amount'];
            $total_descuentos += $detalle['discount_amount'] + $detalle['discount_general_amount'];
            $total_subtotal += $detalle['amount'];
            $total_impuestos += $detalle['taxes_amount'];
            $total += $detalle['total'];
            
            if (isset($taxes_applied[$detalle['taxName']], $taxes_applied)) {
                $taxes_applied[$detalle['taxName']] += $detalle['taxes_amount'];
            } else {
                $taxes_applied[$detalle['taxName']] = $detalle['taxes_amount'];
            }
            
            $listDetalles .= "<tr>  
                <td class='text-left'>                       
                    <a class='btn btn-sm btn-default' onclick='setDetailPurchaseToEdit($array);'><i class='fa fa-pencil'></i></a>
                    <a class='btn btn-sm btn-danger' onclick='deleteDetalles($id);'><i class='fa fa-minus'></i></a>
                </td>
                <td class='text-center'>".$detalle['code']."</td>
                <td>".$detalle['description']."<br/>".nl2br($detalle['description_details'])."</td>                    
                <td class='text-right'>".number_format($detalle['quantity'], 2)."</td>
                <td class='text-right'>".number_format($detalle['cost'], 2)."</td>
                <td class='text-right'>".number_format($detalle['discount'], 2)."</td>
                <td class='text-right'>".$discountGeneral.'% '.number_format($detalle['discount_general_amount'], 2)."</td>
                <td class='text-right'>".number_format($detalle['amount'], 2)."</td>
                <td class='text-right'>".$detalle['taxName'].' '.number_format($detalle['taxes_amount'], 2)."</td>                       
                <td class='text-right'>".number_format($detalle['total'], 2)."</td>  
                </tr>";
        }
        
        if (count($taxes_applied)>0) {
            $total_impuestos_string = '';
            foreach ($taxes_applied as $tax => $amount) {
                $total_impuestos_string .= "<tr>"
                        . "<th colspan='2' class='text-right'>$tax</th>"
                        . "<td id='total_impuestos' class='text-right'>".number_format($amount, 2)."</td>"
                        . "</tr>";
            }
        }
        
        return array('listDetails'=>$listDetalles,
                     'totalItems'=>$cantidadItems,
                     'total_importe'=>$total_importe,
                     'descuento_items'=>$descuentos_item,
                     'descuento_general'=>$descuentos_general,
                     'total_descuentos'=>$total_descuentos,
                     'total_subtotal'=>$total_subtotal,
                     'total_impuestos'=>$total_impuestos_string,
                     'total'=>$total
            );
    }
    
    public function getListPurchaseDetails($tokenForm)
    {
        $compraDetalles = $this->getPurchaseDetails($tokenForm);
        $detalles = $this->listCompraDetalles($compraDetalles);
            
        $json = array(
                'response' => true,
                'purchaseDetails' => $detalles['listDetails'],
                'totalItems'=>number_format($detalles['totalItems'], 2),
                'total_importe'=>$detalles['total_importe'],
                'descuento_items'=>$detalles['descuento_items'],
                'descuento_general'=>$detalles['descuento_general'],
                'total_descuentos'=>number_format($detalles['total_descuentos'], 2),
                'total_subtotal'=>number_format($detalles['total_subtotal'], 2),
                'total_impuestos'=>$detalles['total_impuestos'],
                'total'=>$detalles['total']
            );
            
        return $json;
    }

    public function deleteDetalles(array $options)
    {
        $id = $options['id'];
        $repository = new PurchaseGoodAndServiceDetailsTempRepository();
        $currentData = $repository->getById($id);
        
        if ($repository->delete($id)) {
            $response = true;
            $msj = 'Producto eliminado correctamente.';
        } else {
            $response = null;
            $msj = "No se pudo eliminar producto. Intente nuevamente.";
        }
        
        return $this->getListPurchaseDetails($currentData['token_form']);
    }
    
    public function getDefaultDataProduct($options)
    {
        $repo = new GoodAndServiceCatalogRepository();
        $data = $repo->getById($options['product']);
        
        return array(
            'response'=>true,
            'cost'=>$data['cost'],
            'taxes'=>$data['taxes'],
        );
    }
    
    public function getListLots($options)
    {
        $item = $options['item'];
        
        $items = $this->getLotsLike($item);
        $array = array();
        if ($items) {
            foreach ($items as $item) {
                $array[] = array(
                    'value'=>$item['lot'],
                    'label'=>$item['lot']
                );
            }
        }
        return array(
            'response'=>true,
            'lots'=>$array
        );
    }
    
    public function getVendorMethodPayment($options)
    {
        $vendor = $options['vendor'];
        $date = $options['date'];
        $dueDate = $date;
        
        $repoVendor = new VendorRepository();
        $dataVendor = $repoVendor->getById($vendor);
        
        $paymentTermsRepo = new PaymentTermsRepository();
        $paymentTermsData = $paymentTermsRepo->getById($dataVendor['payment_terms']);
        
        if ($date !== '' && !is_null($date)) {
            $tools = new Tools();
            $date = $tools->setFormatDateToDB($date);
            
            $dueDate = $this->getDueDate($date, $paymentTermsData['days']);
            $dueDate = $tools->setFormatDateToForm($dueDate);
        }
        
        return array(
            'response'=>true,
            'payment_terms'=>$paymentTermsData['id'],
            'due_date'=>$dueDate
        );
    }
    
    public function getListProduct($options)
    {
        $repository = new GoodAndServiceCatalogRepository();
        $item = $options['item'];
        
        $items = $repository->getProductsLike($item);
        $array = array();
        if ($items) {
            foreach ($items as $item) {
                $array[] = array(
                    'value'=>$item['id'],
                    'label'=>$item['description']." (".$item['code'].")",
                    'impuestos'=>$item['taxes']
                );
            }
        }
        return array(
            'response'=>true,
            'products'=>$array
        );
    }
    
    public function getProductByCode($options)
    {
        $repo = new GoodAndServiceCatalogRepository();
        $data = $repo->getByCode($options['code']);
        
        return array(
            'response'=>true,
            'idProduct'=>$data['id']
        );
    }
    
    public function applyGeneralDiscountToItems($options)
    {
        $purchaseGoodAndServiceDetailTemp = new PurchaseGoodAndServiceDetailsTempRepository();
        $rs = $purchaseGoodAndServiceDetailTemp->applyGeneralDiscountToItems($options);
        
        if ($rs) {
            return $this->getListPurchaseDetails($options['token_form']);
        }
    }
}
