<?php
class ShippingAjax extends ShippingRepository
{
    private $idShippingRequest = null;
    
    public function __construct()
    {
        parent::__construct();
    }

    public function getResponse($request, $options)
    {
        return $this->$request($options);
    }
    
    public function setIdShippingRequest($idShippingRequest)
    {
        $this->idShippingRequest = $idShippingRequest;
    }

    public function setShippingDetalles(array $options)
    {
        $this->setIdShippingRequest($options['id_shipping_request']);
        if (isset($options['options'])) {
            $data = array();
            foreach ($options['options'] as $row) {
                $data[$row['name']] = $row['value'];
            }

            $options = $data;
            $producto = $this->getProductoById($options['id_product']);
            $data['descripcion'] = $producto['descripcion'];
            $data['id_shipping_request'] = $this->idShippingRequest;
            
            unset($data['action'],$data['agregar_producto_por_idpallet']);

            if (!$this->insertDetalle($data)) {
                return array(
                    'response'=>false,
                    'msg'=>$this->flashmessenger->getRawMessage()
                );
            }
        } elseif (isset($options['id_pallet'])) {
            $repoInventoty = new InventoryRepository();
            $data = $repoInventoty->getInventoryByPallet($options['id_pallet']);
           
            if ($data) {
                $repoShippingDetalles = new ShippingDetallesTempRepository();
                $mixto = $options['mixto'];
                if (is_array($data) && count($data) > 1 && $mixto === '0') {
                    #si pallet scaneado es mixto, aunque lo scanee en text para 'normales' lo debo insertar como mixo
                    #Por eso obtengo el nextConsecutivePallet
                    $mixto = $repoShippingDetalles->getNextConsecutivoPallet($options['token_form']);
                }
                foreach ($data as $product) {
                    $product['id_pallet_inventory'] = $product['id_pallet'];
                    $product['id_inventory'] = $product['id'];
                    $product['token_form'] = $options['token_form'];
                    
                    $quantity_boxes = $repoShippingDetalles->getAvailableStock($product);
                    $product['quantity_boxes'] = $quantity_boxes;
                    $product['quantity_boxes_per_pallets'] = $quantity_boxes;
                    $product['mixto'] = $mixto;
                    $product['token_form'] = $options['token_form'];
                    $product['id_shipping_request'] = $this->idShippingRequest;
                    
                    if (!$this->insertDetalle($product)) {
                        return array(
                            'response'=>false,
                            'msg'=>$this->flashmessenger->getRawMessage()
                        );
                    }
                }
            } else {
                $this->flashmessenger->addMessage(array('danger'=>'Pallet escaneado no tiene stock o no existe'));
                return array(
                           'response'=>false,
                           'msg'=>$this->flashmessenger->getRawMessage()
                       );
            }
        }
            
        $shippingDetalles = $this->getShippingDetalles($options['token_form']);
        $detalles = $this->listShippingDetalles($shippingDetalles);
        $shippingDetallesMixto = null;
        $detallesMixto = null;

        if ($options['mixto'] !== '0' || $options['mixto'] !== null) {
            $shippingDetallesMixto = $this->getShippingDetallesByMixto($options['mixto'], $options['token_form']);
            $detallesMixto = $this->listShippingDetallesMixto($shippingDetallesMixto);
        }

        $json = array(
            'response' => true,
            'shippingDetalles' => $detalles['listDetalles'],
            'shippingSummary'=>$detalles['listSummary'],
            'cajasTotal'=>$detalles['cajasTotal'],
            'palletsTotal'=>$detalles['palletsTotal'],
            'pesoTotal'=>$detalles['pesoTotal'],
            'request_cajasTotal'=>$detalles['request_cajasTotal'],
            'request_palletsTotal'=>$detalles['request_palletsTotal'],
            'shippingDetallesMixto' => $detallesMixto['listDetalles'],
            'cajasTotalMixto' => $detallesMixto['cajasTotal'],
            'pesoTotalMixto' => $detallesMixto['pesoTotal'],
        );

        return $json;
    }
    
    public function listShippingDetalles($detalles)
    {
        $listDetalles = "";
        $cajasTotal = 0;
        $palletsTotal = 0;
        $pesoTotal = 0;
        $sumaPesoTotal = 0;
        $tokenForm = null;
        
        foreach ($detalles as $detalle) {
            $tokenForm = $detalle['token_form'];
            
            $detalle['quantity_boxes_per_pallets'] = $detalle['quantity_boxes'];
            $array = json_encode($detalle);
            $cajasTotal += $detalle['quantity_boxes'];
            $palletsTotal += $detalle['quantity_pallets'];
            $sumaPesoTotal += $detalle['weight_total'];
            
            $idPalletInventory = str_replace(',', "<br>", $detalle['id_pallet_inventory']);
            $descripcion = str_replace(',', "<br>", $detalle['descripcion']);
            $variety = str_replace(',', "<br>", $detalle['varietyName']);
            $size = str_replace(',', "<br>", $detalle['sizeName']);
            $packing = str_replace(',', "<br>", $detalle['packingName']);
            $quality = str_replace(',', "<br>", $detalle['qualityName']);
            $color = str_replace(',', "<br>", $detalle['colorName']);
            $weight = str_replace(',', "<br>", $detalle['weight_concat']);
            $lot = str_replace(',', "<br>", $detalle['lot_concat']);
            $boxesQuantity = str_replace(',', "<br>", $detalle['quantity_boxes_concat']);
            $palletsQuantity = str_replace(',', "<br>", $detalle['quantity_pallets_concat']);
            
            $id = $detalle['id'];
            $listDetalles .= "<tr>"
                . "<td class='text-left'>                       
                    <a class='btn btn-xs btn-default' onclick='setDetalleShippingToEdit($array,this);' data-mixto='{$detalle['mixto']}' data-controller='Shipping'><i class='fa fa-pencil'></i></a>
                    <a class='btn btn-xs btn-danger' onclick='deleteShippingDetalles($id,this);' data-mixto='{$detalle['mixto']}'><i class='fa fa-trash'></i></a>
                </td>  "
                ."<td class='text-center'>".$detalle['id_pallet']."</td>"
                ."<td class='text-center'>".$idPalletInventory."</td>"
                ."<td class='text-center'>".$lot."</td>"
                ."<td class='text-center'>".$descripcion."</td>"
                ."<td class='text-center'>".$variety."</td>"
                ."<td class='text-center'>".$size."</td>"
                ."<td class='text-center'>".$packing."</td>"
                ."<td class='text-center'>".$quality."</td>"
                ."<td class='text-center'>".$color."</td>"
                ."<td class='text-right'>".$weight."</td>"
                ."<td class='text-right'>".$boxesQuantity."</td>"
                ."<td class='text-right'>".$palletsQuantity."</td>"
                ."</tr>";
        }
        
        $shippingSummary = $this->listShippingSummary($tokenForm);
        
        return array('listDetalles'=>$listDetalles,
                     'cajasTotal'=>$cajasTotal,
                     'palletsTotal'=>$palletsTotal,
                     'pesoTotal'=>$sumaPesoTotal,
                     'listSummary'=>$shippingSummary['listSummary'],
                     'request_cajasTotal'=>$shippingSummary['total_boxes'],
                     'request_palletsTotal'=>$shippingSummary['total_pallets'],);
    }
    
    public function listShippingSummary($tokenForm)
    {
        $listSummary = null;
        $cajasTotal = 0;
        $palletsTotal = 0;
        $summary = $this->getShippingSummary($tokenForm);
        
        $shippingRequestRepo = new ShippingRequestRepository();
        $summaryShippingRequest = $shippingRequestRepo->getShippingRequestSavedSummary($this->idShippingRequest);
        $arrayMerge = array_merge($summaryShippingRequest, $summary);
       
        foreach ($arrayMerge as $keyproduct => $detalle) {
            $tr_class = null;
            $requestBoxes = 0;
            $requestPallets = 0;
            $shippingBoxes = 0;
            $shippingPallets = 0;
           
            if (key_exists($keyproduct, $summary)) {
                $shippingBoxes = $summary[$keyproduct]['quantity_boxes'];
                $shippingPallets = $summary[$keyproduct]['quantity_pallets'];
            }
            
            if (key_exists($keyproduct, $summaryShippingRequest)) {
                $requestBoxes = $summaryShippingRequest[$keyproduct]['quantity_boxes'];
                $requestPallets = $summaryShippingRequest[$keyproduct]['quantity_pallets'];
            } else {
                $tr_class = 'danger';
            }
            
            if ($tr_class == null) {
                if ($requestBoxes != $shippingBoxes) {
                    $tr_class = 'warning';
                } else {
                    $tr_class = 'success';
                }
            }
            
            $cajasTotal += $requestBoxes;
            $palletsTotal += $requestPallets;
            
            $listSummary .= "<tr class='$tr_class'>"
                ."<td class='text-center'>".$detalle['lot']."</td>"
                ."<td class='text-center'>".$detalle['descripcion']."</td>"
                ."<td class='text-center'>".$detalle['varietyName']."</td>"
                ."<td class='text-center'>".$detalle['sizeName']."</td>"
                ."<td class='text-center'>".$detalle['packingName']."</td>"
                ."<td class='text-center'>".$detalle['qualityName']."</td>"
                ."<td class='text-center'>".$detalle['colorName']."</td>"
                ."<td class='text-right'>".$detalle['weight']."</td>"
                ."<td class='text-right'>".$requestBoxes."</td>"
                ."<td class='text-right'>".$requestPallets."</td>"
                ."<td class='text-right'>".$shippingBoxes."</td>"
                ."<td class='text-right'>".$shippingPallets."</td>"
                ."</tr>";
        }
        
        return array(
            'listSummary'=>$listSummary,
            'total_boxes'=>$cajasTotal,
            'total_pallets'=>$palletsTotal
        );
    }
    
    public function getListShippingDetalles($tokenForm)
    {
        $shippingDetalles = $this->getShippingDetalles($tokenForm);
        $detalles = $this->listShippingDetalles($shippingDetalles);

        $json = array(
                'response' => true,
                'shippingDetalles' => $detalles['listDetalles'],
                'shippingSummary'=>$detalles['listSummary'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'pesoTotal'=>$detalles['pesoTotal'],
                'request_cajasTotal'=>$detalles['request_cajasTotal'],
                'request_palletsTotal'=>$detalles['request_palletsTotal'],
            );
        return $json;
    }

    public function deleteDetalles(array $options)
    {
        $this->setIdShippingRequest($options['id_shipping_request']);
        $id = $options['id'];
        $repository = new ShippingDetallesTempRepository();
        $data = $repository->getById($id);
        
        if ($data['mixto']!=='0') {
            $result = $repository->deleteDetalleByMixto($data['mixto'], $data['token_form']);
        } else {
            $result = $repository->delete($id);
        }
        
        if ($result) {
            $response = true;
            $msj = 'Producto eliminado correctamentes.';
        } else {
            $response = null;
            $msj = "No se pudo eliminar producto. Intente nuevamente.";
        }
        
        $shippingDetalles = $this->getShippingDetalles($data['token_form']);
        $detalles = $this->listShippingDetalles($shippingDetalles);
        
        $json = array(
                'response' => $response,
                'message'=>$this->_getTranslation($msj),
                'shippingDetalles'=>$detalles['listDetalles'],
                'shippingSummary'=>$detalles['listSummary'],
                'pesoTotal'=>$detalles['pesoTotal'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'request_cajasTotal'=>$detalles['request_cajasTotal'],
                'request_palletsTotal'=>$detalles['request_palletsTotal'],
            );
        return $json;
    }
    
    public function listShippingDetallesMixto($detalles)
    {
        $listDetalles = "";
        $cajasTotal = 0;
        $pesoTotal = 0;
        $sumaPesoTotal = 0;
        foreach ($detalles as $detalle) {
            $detalle['quantity_boxes_per_pallets'] = $detalle['quantity_boxes'];
            $array = json_encode($detalle);
            $cajasTotal += $detalle['quantity_boxes'];
            $pesoTotal = $detalle['quantity_boxes']*$detalle['weight'];
            $sumaPesoTotal += $pesoTotal;
            $id = $detalle['id'];
            $listDetalles .= "<tr>"
                . "<td class='text-left'>                       
                    <a class='btn btn-xs btn-default' onclick='setDetalleShippingToEditMixto($array);'><i class='fa fa-pencil'></i></a>
                    <a class='btn btn-xs btn-danger' onclick='deleteShippingDetallesMixto($id);'><i class='fa fa-trash'></i></a>
                </td>  "
               ."<td class='text-center'>".$detalle['descripcion']."<br/>".$detalle['varietyName']."</td>"
               ."<td class='text-center'>".$detalle['sizeName']."</td>"
               ."<td class='text-center'>".$detalle['brandName']."</td>"
               ."<td class='text-center'>".$detalle['qualityName']."</td>"
               ."<td class='text-right'>".$detalle['quantity_boxes']."</td>"
               ."</tr>";
        }
        
        return array('listDetalles'=>$listDetalles,
                     'cajasTotal'=>$cajasTotal,
                     'pesoTotal'=>$sumaPesoTotal);
    }
    
    public function getListShippingDetallesMixto($options)
    {
        $shippingDetallesMixto = $this->getShippingDetallesByMixto($options['mixto'], $options['token_form']);
        $detallesMixto = $this->listShippingDetallesMixto($shippingDetallesMixto);
        
        $json = array(
            'response' => true,
            'shippingDetallesMixto' => $detallesMixto['listDetalles'],
            'cajasTotalMixto' => $detallesMixto['cajasTotal'],
            'pesoTotalMixto' => $detallesMixto['pesoTotal'],
        );
        
        return $json;
    }
    
    public function deleteDetallesMixto(array $options)
    {
        $id = $options['id'];
        $repository = new ShippingDetallesTempRepository();
        $currentData = $repository->getById($id);
        
        if ($repository->delete($id)) {
            $repository->assignMainProductForMixto($currentData['mixto'], $currentData['token_form']);
            $response = true;
            $msj = 'Producto eliminado correctamentes.';
        } else {
            $response = null;
            $msj = "No se pudo eliminar producto. Intente nuevamente.";
        }
        
        $shippingDetalles = $this->getShippingDetalles($currentData['token_form']);
        $detalles = $this->listShippingDetalles($shippingDetalles);
        
        $dataMixto = $this->getListShippingDetallesMixto(array('mixto'=>$currentData['mixto'],'token_form'=>$currentData['token_form']));
        $json = array(
                'response' => $response,
                'message'=>$this->_getTranslation($msj),
                'shippingDetalles'=>$detalles['listDetalles'],
                'shippingSummary'=>$detalles['listSummary'],
                'pesoTotal'=>$detalles['pesoTotal'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'request_cajasTotal'=>$detalles['request_cajasTotal'],
                'request_palletsTotal'=>$detalles['request_palletsTotal'],
                'shippingDetallesMixto' => $dataMixto['shippingDetallesMixto'],
                'cajasTotalMixto' => $dataMixto['cajasTotalMixto'],
                'pesoTotalMixto' => $dataMixto['pesoTotalMixto'],
            );
        return $json;
    }
    
    public function getDataShippingRequest($options)
    {
        $this->setIdShippingRequest($options['id_shipping_request']);
        $repoShippingTempDetails = new ShippingDetallesTempRepository();
        $repoShippingTempDetails->truncate($options['token_form']);
        
        $entityShippingRequest = new ShippingRequestRepository();
        $infoOutbound = $entityShippingRequest->getById($options['id_shipping_request']);
                
        if ($infoOutbound) {
            if ($infoOutbound['status'] != 3) {
                $entityShippingRequest->setOptions($infoOutbound);
            
                $direccionEntrega = $entityShippingRequest->getDataShippingAddress();
                $stringDireccionEntrega = '';
                $stringDireccionEntrega .= $direccionEntrega['address']."<br>";
                $stringDireccionEntrega .= $direccionEntrega['city'].",";
                $stringDireccionEntrega .= $direccionEntrega['state']." ";
                $stringDireccionEntrega .= $direccionEntrega['zipcode']."<br>";

                $infoOutbound['shipping_address'] = $stringDireccionEntrega;
                
                $listDetails = $this->listShippingSummary($options['token_form']);

                return array(
                    'response'=>true,
                    'infoOutbound'=>$infoOutbound,
                    'shippingSummary'=>$listDetails['listSummary']
                );
            } else {
                return array(
                        'response'=>false,
                        'message'=>$this->_getTranslation('Solicutd de Embarque cancelada.')
                    );
            }
        }
        
        return array(
                'response'=>false,
                'message'=>$this->_getTranslation('Solicutd de Embarque no existe en el sistema.')
            );
    }
    
    public function setDataShippingRequest($options)
    {
        $entityShippingRequest = new ShippingRequestRepository();
        $infoOutbound = $entityShippingRequest->getById($options['id_shipping_request']);
                
        if ($infoOutbound) {
            if ($infoOutbound['status'] != 3) {
                $entityShippingRequest->setOptions($infoOutbound);
            
                $direccionEntrega = $entityShippingRequest->getDataShippingAddress();
                $stringDireccionEntrega = '';
                $stringDireccionEntrega .= $direccionEntrega['address']."<br>";
                $stringDireccionEntrega .= $direccionEntrega['city'].",";
                $stringDireccionEntrega .= $direccionEntrega['state']." ";
                $stringDireccionEntrega .= $direccionEntrega['zipcode']."<br>";

                $infoOutbound['shipping_address'] = $stringDireccionEntrega;

                return array(
                    'response'=>true,
                    'infoOutbound'=>$infoOutbound
                );
            } else {
                return array(
                        'response'=>false,
                        'message'=>$this->_getTranslation('Salida cancelada.')
                    );
            }
        }
        
        return array(
                'response'=>false,
                'message'=>$this->_getTranslation('Solicutd de Embarque no existe en el sistema')
            );
    }
    
    public function getAvailableInventoryByPallet($options)
    {
        $idProduct = $options['id_product'];
        
        $repoProduct = new ProductRepository();
        $dataProduct = $repoProduct->getById($idProduct);
        $message = "<h4>".$this->_getTranslation('Stock disponible para')." ".$dataProduct['descripcion']."</h4>";
        
        $repoInventory = new InventoryRepository();
        $availableInventory = $repoInventory->getSummaryInventoryByPallet($idProduct);
        
        if ($availableInventory === null) {
            $message .= "<p>".$this->_getTranslation('No hay existencias para este producto.')."</p>";
            $json = array(
                 'response'=>false,
                 'availableInventory'=>null,
                 'message'=>$message);
            
            return $json;
        }
        
        $string = "";
        foreach ($availableInventory as $producto) {
            $infoProducto = json_encode($producto);
            
            #setReceivingDetail funcion generica para popular formularios con info enviada. nombrecampo=>valor
            $string .= "<tr style='cursor:pointer' onclick='setInfoProductShipping($infoProducto)'>";
     
            $string .=  "<td style='text-align:center'>{$producto['id_pallet']}</td>"
                    . "     <td style='text-align:center'>{$producto['date']}</td>
                            <td style='text-align:center'>{$producto['lot']}</td>
                            <td style='text-align:center'>{$producto['descripcion']}</td>
                            <td style='text-align:center'>{$producto['varietyName']}</td>
                            <td style='text-align:center'>{$producto['madurityName']}</td>
                            <td style='text-align:center'>{$producto['sizeName']}</td>
                            <td style='text-align:center'>{$producto['packingName']}</td>
                            <td style='text-align:center'>{$producto['brandName']}</td>
                            <td style='text-align:center'>{$producto['qualityName']}</td>                                
                            <td style='text-align:center'>{$producto['colorName']}</td>
                            <td style='text-align:center'>{$producto['weight']}</td>
                            <td style='text-align:center'>{$producto['quantity_boxes']}</td>
                       </tr>";
        }
        
        $json = array(
                 'response'=>true,
                 'availableInventory'=>$string,
                 'message'=>$message);
        return $json;
    }
    
    public function allowEditShipping($options)
    {
        $entityRepository = new EntityRepository();
        $entityRepository->update($options['idShipping'], array('allow_edit'=>'1'), 'shipping');
        return array(
            'response'=>true,
        );
    }
    
    public function matchRequestVsShipping($options)
    {
        $repoShippingRequest = new ShippingRequestRepository();
        $request = $repoShippingRequest->getShippingRequestSavedSummary($options['id_shipping_request']);
        
        $shipping = $this->getShippingSummary($options['token_form']);
        
        foreach ($shipping as $keyProduct => $data) {
            if (key_exists($keyProduct, $request)) {
                if ($data['quantity_boxes'] != $request[$keyProduct]['quantity_boxes']) {
                    return array('response'=>null);
                }
            } else {
                return array('response'=>null);
            }
        }
        
        foreach ($request as $keyProduct => $data) {
            if (key_exists($keyProduct, $shipping)) {
                if ($data['quantity_boxes'] != $request[$keyProduct]['quantity_boxes']) {
                    return array('response'=>null);
                }
            } else {
                return array('response'=>null);
            }
        }
        
        return array('response'=>true);
    }
    
    /*OVERWEIGHT*/
    public function setOverWeightDetalles(array $options)
    {
        $overWeightDetallesTemp = new ShippingOverWeightDetallesTempRepository();
        $rs = $overWeightDetallesTemp->setOverWeightByIdPallet($options['id_shipping'], $options['id_pallet'], $options['token_form']);
      
        if (!$rs) {
            return array(
                'response'=>false,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
            
        $shippingDetalles = $overWeightDetallesTemp->getOverWeightDetalles($options['token_form']);
        $detalles = $this->listOverWeightDetalles($shippingDetalles);

        $json = array(
            'response' => true,
            'overWeightDetalles' => $detalles['listDetalles'],
            'cajasTotal'=>$detalles['cajasTotal'],
            'palletsTotal'=>$detalles['palletsTotal'],
            'pesoTotal'=>$detalles['pesoTotal'],
        );

        return $json;
    }
    
    public function listOverWeightDetalles($detalles)
    {
        $listDetalles = "";
        $cajasTotal = 0;
        $palletsTotal = 0;
        $pesoTotal = 0;
        $sumaPesoTotal = 0;
        $tokenForm = null;
        
        foreach ($detalles as $detalle) {
            $tokenForm = $detalle['token_form'];
            
            $detalle['quantity_boxes_per_pallets'] = $detalle['quantity_boxes'];
            $array = json_encode($detalle);
            $cajasTotal += $detalle['quantity_boxes'];
            $palletsTotal += $detalle['quantity_pallets'];
            $sumaPesoTotal += $detalle['weight'];
            
            $idPalletInventory = str_replace(',', "<br>", $detalle['id_pallet_inventory']);
            $descripcion = str_replace(',', "<br>", $detalle['descripcion']);
            $variety = str_replace(',', "<br>", $detalle['varietyName']);
            $size = str_replace(',', "<br>", $detalle['sizeName']);
            $packing = str_replace(',', "<br>", $detalle['packingName']);
            $quality = str_replace(',', "<br>", $detalle['qualityName']);
            $color = str_replace(',', "<br>", $detalle['colorName']);
            $weight = str_replace(',', "<br>", $detalle['weight']);
            $boxesQuantityOrigin = str_replace(',', "<br>", $detalle['quantity_boxes_origin']);
            $boxesQuantity = str_replace(',', "<br>", $detalle['quantity_boxes']);
            $palletsQuantity = str_replace(',', "<br>", $detalle['quantity_pallets']);
            
            $id = $detalle['id'];
            $listDetalles .= "<tr>"
                . "<td class='text-left'>                                           
                    <a class='btn btn-xs btn-danger' onclick='deleteOverWeightDetalles($id,this);' data-mixto='{$detalle['mixto']}'><i class='fa fa-trash'></i></a>
                </td>  "
                ."<td class='text-center'>".$detalle['id_pallet']."</td>"
                ."<td class='text-center'>".$idPalletInventory."</td>"
                ."<td class='text-center'>".$descripcion."</td>"
                ."<td class='text-center'>".$variety."</td>"
                ."<td class='text-center'>".$size."</td>"
                ."<td class='text-center'>".$packing."</td>"
                ."<td class='text-center'>".$quality."</td>"
                ."<td class='text-center'>".$color."</td>"
                ."<td class='text-right'>".$boxesQuantityOrigin."</td>";
            
            if ($detalle['saved'] == '1') {
                $listDetalles .= "<td class='text-right'>{$boxesQuantity}</td>";
            } else {
                $listDetalles .= "<td class='text-center'><input name='{$detalle['id']}' type='text' value='{$boxesQuantity}' class='_overWeightQuantity text-right' /></td>";
            }
            
            $listDetalles .= "<td class='text-right'>".$weight."</td>"   ;
            $listDetalles .= "</tr>";
        }
        
        return array('listDetalles'=>$listDetalles,
                     'cajasTotal'=>$cajasTotal,
                     'palletsTotal'=>$palletsTotal,
                     'pesoTotal'=>$sumaPesoTotal);
    }
    
    public function getListOverWeightDetalles($tokenForm)
    {
        $overWeightDetallesTemp = new ShippingOverWeightDetallesTempRepository();
        $shippingDetalles = $overWeightDetallesTemp->getOverWeightDetalles($tokenForm);
        $detalles = $this->listOverWeightDetalles($shippingDetalles);

        $json = array(
                'response' => true,
                'overWeightDetalles' => $detalles['listDetalles'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'pesoTotal'=>$detalles['pesoTotal'],
            );
        return $json;
    }
    
    public function updateOverWeightQuantity($options)
    {
        $tokenForm = $options['token_form'];
        $repository = new ShippingOverWeightDetallesTempRepository();
        $shippingDetailsRepo = new ShippingDetallesTempRepository();
        try {
            if (isset($options['overweight_quantity'])) {
                $data = array();
                foreach ($options['overweight_quantity'] as $row) {
                    $data[$row['name']] = $row['value'];
                }
  
                $repository->startTransaction();

                /*ACTUALIZAR TBL OVERWEIGHT*/
                foreach ($data as $id => $quantity) {
                    $currentData = $repository->getById($id);

                    if ($quantity > $currentData['quantity_boxes_origin']) {
                        $this->flashmessenger->addMessage(array('danger'=>'Cantidad a descargar no puede ser mayor a la cantidad de cajas en pallet.'));
                        return array('response'=>false,'msg'=>$this->flashmessenger->getRawMessage());
                    }
                    //echo "UPDATE ".$repository->getTableName()." SET quantity_boxes = '$quantity' WHERE id = '$id' AND token_form = '$tokenForm'";
                    $rs = parent::query("UPDATE ".$repository->getTableName()." SET quantity_boxes = '$quantity' WHERE id = '$id' AND token_form = '$tokenForm'");
                }

                $repository->commit();

                /*ACTUALIZAR TBL SHIPPING_DETAILS_X*/
                $shippingDetailsRepo->startTransaction();

                foreach ($data as $id => $quantity) {
                    $currentData = $repository->getById($id);
                    $result = $shippingDetailsRepo->query("SELECT * FROM {$shippingDetailsRepo->getTableName()} WHERE id_detail = '{$currentData['id_detail']}' AND token_form = '$tokenForm' ");
                    $currentShippingDetailsData = $shippingDetailsRepo->resultToArray($result)[0];

                    if ($quantity == $currentShippingDetailsData['quantity_boxes']) {
                        /*Significa que descargaron todo el producto*/
                        /*Se elimina producto de shipping*/
                        $shippingDetailsRepo->delete($currentShippingDetailsData['id']);
                    } elseif ($quantity > 0) {
                        /*Se bajo parte del producto*/
                        $currentShippingDetailsData['idDetalleTemp'] = $currentShippingDetailsData['id'];
                        $currentShippingDetailsData['quantity_boxes'] = $currentShippingDetailsData['quantity_boxes'] - $quantity;

                        $shippingDetailsRepo->updateTemp($currentShippingDetailsData);
                    }
                }

                $shippingDetailsRepo->commit();
            }
            
            /*DELETE FROM TBL_SHIPPING_OVERWEIGHT_DETAILS*/
            /*OBTENEMOS DETALLES ORIGINALES DE OVERWEIGHT*/
            $query = "SELECT * FROM shipping_overweight_details WHERE id_shipping = '{$options['id_shipping']}'";
            $result = $repository->query($query);
            $detallesOrigin = null;
            if ($result->num_rows > 0) {
                $detallesOrigin = $repository->resultToArray($result);
            }
            
            /*OBTENEMOS DETALLES DE OVERWEIGHT DESPUES DE EDITAR(Solo se pueden eliminar)*/
            $query = "SELECT * FROM {$repository->getTableName()} WHERE token_form = '$tokenForm'";
            $result = $repository->query($query);
            $idsDetalles = array();
            if ($result->num_rows > 0) {
                $result = $repository->resultToArray($result);
                foreach ($result as $row) {
                    $idsDetalles[] = $row['id_detail'];
                }
            }

            /*ELIMINAMOS DETALLES QUE YA NO ESTAN DESPUES DE EDITAR (que no estan en $idDetalles)*/
            if ($detallesOrigin) {
                $entityRepository = new EntityRepository();
                foreach ($detallesOrigin as $detalle) {
                    if (!in_array($detalle['id'], $idsDetalles)) {
                        $currentData = $entityRepository->getById($detalle['id'], 'shipping_overweight_details');
                        $entityRepository->delete($detalle['id'], 'shipping_overweight_details');
                        
                        $query = "SELECT * FROM {$shippingDetailsRepo->getTableName()} WHERE id_detail = '{$currentData['id_shipping_detail']}' AND token_form = '$tokenForm'";
                        $result = $shippingDetailsRepo->query($query);
                        
                        if ($result->num_rows > 0) {
                            $result = $shippingDetailsRepo->resultToArray($result)[0];
                            $quanity_boxes = $currentData['quantity_boxes'];

                            $query = "UPDATE {$shippingDetailsRepo->getTableName()} SET quantity_boxes = quantity_boxes + $quanity_boxes WHERE id = '{$result['id']}'";
                            $shippingDetailsRepo->query($query);
                        } else {
                            $currentData['token_form'] = $tokenForm;
                            $currentData['quantity_boxes_per_pallets'] = $currentData['quantity_boxes'];
                            $shippingDetailsRepo->saveTemp($currentData);
                        }
                    }
                }
            }
            
            /*ACTUALIZAR TBL_SHIPPING_DETAILS*/
            /*Aqui se actuliza lo que se tenga que actuliar en inventarios*/
            $shippingDetailsRepo->updateDetalles($tokenForm, $options['id_shipping'], '3', '3');
            
            /*INSERTAR EN TBL_SHIPPING_OVERWEIGHT_DETAILS*/
            $repository->saveDetalles($tokenForm);
            return array('response'=>true);
        } catch (Exception $ex) {
            $error = $ex->getMessage();
            //$this->flashmessenger->addMessage(array('danger'=>'Error al registrar sobrepeso. Intentelo nuevamente.'));
            $this->flashmessenger->addMessage(array('danger'=>$error));
            return array('response'=>false,'msg'=>$this->flashmessenger->getRawMessage());
        }
    }
    
    public function deleteOverWeightDetalles($options)
    {
        $repository = new ShippingOverWeightDetallesTempRepository();
        $currentData = $repository->getById($options['id']);
        $result = $repository->delete($options['id']);
        
        if ($result) {
            $response = true;
            $msj = 'Producto eliminado correctamentes.';
        } else {
            $response = null;
            $msj = "No se pudo eliminar producto. Intente nuevamente.";
        }
        
        $detalles = $this->getListOverWeightDetalles($currentData['token_form']);
        
        $json = array(
                'response' => $response,
                'message'=>$this->_getTranslation($msj),
                'overWeightDetalles' => $detalles['overWeightDetalles'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'pesoTotal'=>$detalles['pesoTotal'],
            );
        return $json;
    }
    /* END OVERWEIGHT*/
    
    /*RETURNS*/
    public function setReturnDetalles(array $options)
    {
        $returnDetallesTemp = new ShippingReturnDetallesTempRepository();
        $rs = $returnDetallesTemp->setReturnByIdPallet($options['id_shipping'], $options['id_pallet'], $options['token_form']);
      
        if (!$rs) {
            return array(
                'response'=>false,
                'msg'=>$this->flashmessenger->getRawMessage()
            );
        }
            
        $shippingDetalles = $returnDetallesTemp->getReturnDetalles($options['token_form']);
        $detalles = $this->listReturnDetalles($shippingDetalles);

        $json = array(
            'response' => true,
            'returnDetalles' => $detalles['listDetalles'],
            'cajasTotal'=>$detalles['cajasTotal'],
            'palletsTotal'=>$detalles['palletsTotal'],
            'pesoTotal'=>$detalles['pesoTotal'],
        );

        return $json;
    }
    
    public function listReturnDetalles($detalles)
    {
        $listDetalles = "";
        $cajasTotal = 0;
        $palletsTotal = 0;
        $pesoTotal = 0;
        $sumaPesoTotal = 0;
        $tokenForm = null;
        
        foreach ($detalles as $detalle) {
            $tokenForm = $detalle['token_form'];
            
            $detalle['quantity_boxes_per_pallets'] = $detalle['quantity_boxes'];
            $array = json_encode($detalle);
            $cajasTotal += $detalle['quantity_boxes'];
            $palletsTotal += $detalle['quantity_pallets'];
            $sumaPesoTotal += $detalle['weight'];
            
            $idPalletInventory = str_replace(',', "<br>", $detalle['id_pallet_inventory']);
            $descripcion = str_replace(',', "<br>", $detalle['descripcion']);
            $variety = str_replace(',', "<br>", $detalle['varietyName']);
            $size = str_replace(',', "<br>", $detalle['sizeName']);
            $packing = str_replace(',', "<br>", $detalle['packingName']);
            $quality = str_replace(',', "<br>", $detalle['qualityName']);
            $color = str_replace(',', "<br>", $detalle['colorName']);
            $weight = str_replace(',', "<br>", $detalle['weight']);
            $boxesQuantityOrigin = str_replace(',', "<br>", $detalle['quantity_boxes_origin']);
            $boxesQuantity = str_replace(',', "<br>", $detalle['quantity_boxes']);
            $palletsQuantity = str_replace(',', "<br>", $detalle['quantity_pallets']);
            
            $id = $detalle['id'];
            $listDetalles .= "<tr>"
                . "<td class='text-left'>                                           
                    <a class='btn btn-xs btn-danger' onclick='deleteReturnDetalles($id,this);' data-mixto='{$detalle['mixto']}'><i class='fa fa-trash'></i></a>
                </td>  "
                ."<td class='text-center'>".$detalle['id_pallet']."</td>"
                ."<td class='text-center'>".$idPalletInventory."</td>"
                ."<td class='text-center'>".$descripcion."</td>"
                ."<td class='text-center'>".$variety."</td>"
                ."<td class='text-center'>".$size."</td>"
                ."<td class='text-center'>".$packing."</td>"
                ."<td class='text-center'>".$quality."</td>"
                ."<td class='text-center'>".$color."</td>"
                ."<td class='text-right'>".$boxesQuantityOrigin."</td>";
            
            if ($detalle['saved'] == '1') {
                $listDetalles .= "<td class='text-right'>{$boxesQuantity}</td>";
            } else {
                $listDetalles .= "<td class='text-center'><input name='{$detalle['id']}' type='text' value='{$boxesQuantity}' class='_returnQuantity text-right' /></td>";
            }
            
            $listDetalles .= "<td class='text-right'>".$weight."</td>"   ;
            $listDetalles .= "</tr>";
        }
        
        return array('listDetalles'=>$listDetalles,
                     'cajasTotal'=>$cajasTotal,
                     'palletsTotal'=>$palletsTotal,
                     'pesoTotal'=>$sumaPesoTotal);
    }
    
    public function getListReturnDetalles($tokenForm)
    {
        $returnDetallesTemp = new ShippingReturnDetallesTempRepository();
        $shippingDetalles = $returnDetallesTemp->getReturnDetalles($tokenForm);
        $detalles = $this->listReturnDetalles($shippingDetalles);

        $json = array(
                'response' => true,
                'returnDetalles' => $detalles['listDetalles'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'pesoTotal'=>$detalles['pesoTotal'],
            );
        return $json;
    }
    
    public function updateReturnQuantity($options)
    {
        $tokenForm = $options['token_form'];
        $repository = new ShippingReturnDetallesTempRepository();
        $shippingDetailsRepo = new ShippingDetallesTempRepository();
        try {
            if (isset($options['return_quantity'])) {
                $data = array();
                foreach ($options['return_quantity'] as $row) {
                    $data[$row['name']] = $row['value'];
                }
  
                $repository->startTransaction();

                /*ACTUALIZAR TBL RETURNS*/
                foreach ($data as $id => $quantity) {
                    $currentData = $repository->getById($id);

                    if ($quantity > $currentData['quantity_boxes_origin']) {
                        $this->flashmessenger->addMessage(array('danger'=>'Cantidad a retornar no puede ser mayor a la cantidad de cajas en pallet.'));
                        return array('response'=>false,'msg'=>$this->flashmessenger->getRawMessage());
                    }
                    //echo "UPDATE ".$repository->getTableName()." SET quantity_boxes = '$quantity' WHERE id = '$id' AND token_form = '$tokenForm'";
                    $rs = parent::query("UPDATE ".$repository->getTableName()." SET quantity_boxes = '$quantity' WHERE id = '$id' AND token_form = '$tokenForm'");
                }

                $repository->commit();

                /*ACTUALIZAR TBL SHIPPING_DETAILS_X*/
                $shippingDetailsRepo->startTransaction();

                foreach ($data as $id => $quantity) {
                    $currentData = $repository->getById($id);
                    $result = $shippingDetailsRepo->query("SELECT * FROM {$shippingDetailsRepo->getTableName()} WHERE id_detail = '{$currentData['id_detail']}' AND token_form = '$tokenForm' ");
                    $currentShippingDetailsData = $shippingDetailsRepo->resultToArray($result)[0];

                    if ($quantity == $currentShippingDetailsData['quantity_boxes']) {
                        /*Significa que retornan todo el producto*/
                        /*Se elimina producto de shipping*/
                        $shippingDetailsRepo->delete($currentShippingDetailsData['id']);
                    } elseif ($quantity > 0) {
                        /*Se bajo parte del producto*/
                        $currentShippingDetailsData['idDetalleTemp'] = $currentShippingDetailsData['id'];
                        $currentShippingDetailsData['quantity_boxes'] = $currentShippingDetailsData['quantity_boxes'] - $quantity;

                        $shippingDetailsRepo->updateTemp($currentShippingDetailsData);
                    }
                }

                $shippingDetailsRepo->commit();
            }
            
            /*DELETE FROM TBL_SHIPPING_RETURN_DETAILS*/
            /*OBTENEMOS DETALLES ORIGINALES DE RETURN*/
            $query = "SELECT * FROM shipping_return_details WHERE id_shipping = '{$options['id_shipping']}'";
            $result = $repository->query($query);
            $detallesOrigin = null;
            if ($result->num_rows > 0) {
                $detallesOrigin = $repository->resultToArray($result);
            }
            
            /*OBTENEMOS DETALLES DE RETURN DESPUES DE EDITAR(Solo se pueden eliminar)*/
            $query = "SELECT * FROM {$repository->getTableName()} WHERE token_form = '$tokenForm'";
            $result = $repository->query($query);
            $idsDetalles = array();
            if ($result->num_rows > 0) {
                $result = $repository->resultToArray($result);
                foreach ($result as $row) {
                    $idsDetalles[] = $row['id_detail'];
                }
            }

            /*ELIMINAMOS DETALLES QUE YA NO ESTAN DESPUES DE EDITAR (que no estan en $idDetalles)*/
            if ($detallesOrigin) {
                $entityRepository = new EntityRepository();
                foreach ($detallesOrigin as $detalle) {
                    if (!in_array($detalle['id'], $idsDetalles)) {
                        $currentData = $entityRepository->getById($detalle['id'], 'shipping_return_details');
                        $entityRepository->delete($detalle['id'], 'shipping_return_details');
                        
                        $query = "SELECT * FROM {$shippingDetailsRepo->getTableName()} WHERE id_detail = '{$currentData['id_shipping_detail']}' AND token_form = '$tokenForm'";
                        $result = $shippingDetailsRepo->query($query);
                        
                        if ($result->num_rows > 0) {
                            $result = $shippingDetailsRepo->resultToArray($result)[0];
                            $quanity_boxes = $currentData['quantity_boxes'];

                            $query = "UPDATE {$shippingDetailsRepo->getTableName()} SET quantity_boxes = quantity_boxes + $quanity_boxes WHERE id = '{$result['id']}'";
                            $shippingDetailsRepo->query($query);
                        } else {
                            $shippingRepo = new ShippingRepository();
                            $shippingData = $shippingRepo->getById($currentData['id_shipping']);
                            
                            $currentData['id_shipping_request'] = $shippingData['id_shipping_request'];
                            $currentData['token_form'] = $tokenForm;
                            $currentData['quantity_boxes_per_pallets'] = $currentData['quantity_boxes'];
                            $shippingDetailsRepo->saveTemp($currentData);
                        }
                    }
                }
            }
            
            /*ACTUALIZAR TBL_SHIPPING_DETAILS*/
            /*Aqui se actuliza lo que se tenga que actuliar en inventarios*/
            $shippingDetailsRepo->updateDetalles($tokenForm, $options['id_shipping'], '3', '3');
            
            /*INSERTAR EN TBL_SHIPPING_RETURN_DETAILS*/
            $repository->saveDetalles($tokenForm);
            return array('response'=>true);
        } catch (Exception $ex) {
            $error = $ex->getMessage();
            //$this->flashmessenger->addMessage(array('danger'=>'Error al registrar sobrepeso. Intentelo nuevamente.'));
            $this->flashmessenger->addMessage(array('danger'=>$error));
            return array('response'=>false,'msg'=>$this->flashmessenger->getRawMessage());
        }
    }
    
    public function deleteReturnDetalles($options)
    {
        $repository = new ShippingReturnDetallesTempRepository();
        $currentData = $repository->getById($options['id']);
        $result = $repository->delete($options['id']);
        
        if ($result) {
            $response = true;
            $msj = 'Producto eliminado correctamentes.';
        } else {
            $response = null;
            $msj = "No se pudo eliminar producto. Intente nuevamente.";
        }
        
        $detalles = $this->getListReturnDetalles($currentData['token_form']);
        
        $json = array(
                'response' => $response,
                'message'=>$this->_getTranslation($msj),
                'returnDetalles' => $detalles['returnDetalles'],
                'cajasTotal'=>$detalles['cajasTotal'],
                'palletsTotal'=>$detalles['palletsTotal'],
                'pesoTotal'=>$detalles['pesoTotal'],
            );
        return $json;
    }
    /* END RETURN*/
    
    /*EMAILING BOL*/
    public function getDataShipping($options)
    {
        $shippingData = $this->getById($options['id_shipping']);
        $this->setOptions($shippingData);
        
        $customerRepo = new CustomerRepository();
        $customerData = $customerRepo->getById($this->getCustomer());
        
        return array(
            'response'=>true,
            'to'=>$customerData['correo'],
            'subject'=>$this->_getTranslation("Bill of loading # {$shippingData['id_shipping_request']}"),
            'messageMail'=>$this->_getTranslation("Revisar archivos adjuntos porfavor."),
        );
    }
    
    public function emailingBOL($options)
    {
        $data = array();
        foreach ($options['options'] as $row) {
            $data[$row['name']] = $row['value'];
        }
        
        $shippingRepo = new ShippingRepository();
        $shippingRepo->setOptions($shippingRepo->getById($data['id_shipping']));
        $shippingRepo->setId($data['id_shipping']);
        $bolPdf = $shippingRepo->crearPDF();
        
        $company = new CompanyRepository();
        $company->setOptions($company->getById(1));
       
        try {
            $emailer = new Emailer();
            $emailer->sendEmail(array(
                'to'=>$data['to'],
                'cc'=>$data['cc'],
                'subject'=>$data['subject'],
                'from_title'=>$company->getNombre(),
                'message'=>$data['message'],
                'attachment'=>$bolPdf
            ));
            
            $this->flashmessenger->addMessage(array('success'=>'EL BOL se envio correctamente.'));
            return array(
                'response'=>true,
                'msg'=>$this->flashmessenger->getMessageString()
            );
        } catch (Exception $ex) {
            $this->flashmessenger->addMsg(array('danger'=>'Oops =(. Algo salio mal al tratar de enviar el BOL. Intenta nuevamente'));
            return array(
                'response'=>null,
                'msg'=>$this->flashmessenger->getMessageString()
            );
        }
    }
    /*EN EMAILING BOL*/
}
